@@ -2,7 +2,7 @@
 # encoding: utf-8
 
 __author__ = "ChenyangGao <https://chenyanggao.github.io>"
-__version__ = (0, 0, 10)
+__version__ = (0, 1, 0)
 __all__ = [
     "thread_batch", "thread_pool_batch", "async_batch", 
     "threaded", "run_as_thread", "asynchronized", "run_as_async", 
@@ -27,7 +27,7 @@ from threading import Event, Lock, Semaphore, Thread
 from typing import cast, overload, Any, ContextManager, Literal
 
 from argtools import argcount
-from asynctools import async_zip, run_async
+from asynctools import async_map, async_zip, run_async
 from decotools import optional
 
 
@@ -285,37 +285,48 @@ def threadpool_map[T](
 ) -> Iterator[T]:
     if max_workers is None or max_workers <= 0:
         max_workers = min(32, (cpu_count() or 1) + 4)
-    queue: SimpleQueue[None | Future] = SimpleQueue()
-    get, put = queue.get, queue.put_nowait
-    executor = ThreadPoolExecutor(max_workers=max_workers)
-    submit = executor.submit
-    running = True
-    def make_tasks():
-        try:
+    if max_workers == 1:
+        if arg_func is None:
+            yield from map(partial(func, **kwargs), it, *its)
+        else:
             for args in zip(it, *its):
-                if not running:
-                    break
-                try:
-                    if arg_func is None:
-                        put(submit(func, *args, **kwargs))
-                    else:
-                        arg = arg_func(*args, **kwargs)
-                        if isinstance(arg, Return):
-                            put(arg.value)
-                            continue
-                        put(submit(func, *args, arg, **kwargs))
-                except RuntimeError:
-                    break
+                arg = arg_func(*args, **kwargs)
+                if isinstance(arg, Return):
+                    yield arg.value
+                    continue
+                yield func(*args, arg, **kwargs)
+    else:
+        queue: SimpleQueue[None | Future] = SimpleQueue()
+        get, put = queue.get, queue.put_nowait
+        executor = ThreadPoolExecutor(max_workers=max_workers)
+        submit = executor.submit
+        running = True
+        def make_tasks():
+            try:
+                for args in zip(it, *its):
+                    if not running:
+                        break
+                    try:
+                        if arg_func is None:
+                            put(submit(func, *args, **kwargs))
+                        else:
+                            arg = arg_func(*args, **kwargs)
+                            if isinstance(arg, Return):
+                                put(arg.value)
+                                continue
+                            put(submit(func, *args, arg, **kwargs))
+                    except RuntimeError:
+                        break
+            finally:
+                put(None)
+        try:
+            start_new_thread(make_tasks, ())
+            with executor:
+                while fu := get():
+                    yield fu.result()
         finally:
-            put(None)
-    try:
-        start_new_thread(make_tasks, ())
-        with executor:
-            while fu := get():
-                yield fu.result()
-    finally:
-        running = False
-        executor.shutdown(wait=False, cancel_futures=True)
+            running = False
+            executor.shutdown(wait=False, cancel_futures=True)
 
 
 async def taskgroup_map[T](
@@ -329,34 +340,51 @@ async def taskgroup_map[T](
 ) -> AsyncIterator[T]:
     if max_workers is None or max_workers <= 0:
         max_workers = 32
-    sema = AsyncSemaphore(max_workers)
-    queue: AsyncQueue[None | AsyncFuture] = AsyncQueue()
-    get, put = queue.get, queue.put_nowait
-    async def call(args):
-        async with sema:
-            ret = func(*args, **kwargs)
-            if isawaitable(ret):
-                ret = await ret
-            return ret
-    async def make_tasks():
-        try:
+    if max_workers == 1:
+        if arg_func is None:
+            async for ret in async_map(partial(func, **kwargs), it, *its):
+                yield cast(T, ret)
+        else:
             async for args in async_zip(it, *its):
-                if arg_func is not None:
-                    arg = arg_func(*args, **kwargs)
-                    if isawaitable(arg):
-                        arg = await arg
-                    if isinstance(arg, Return):
-                        put(arg.value)
-                        continue
-                    args = (*args, arg)
-                put(create_task(call(args)))
-        finally:
-            put(None)
-    async with TaskGroup() as tg:
-        create_task = tg.create_task
-        create_task(make_tasks())
-        while task := await get():
-            yield await task
+                arg = arg_func(*args, **kwargs)
+                if isawaitable(arg):
+                    arg = await arg
+                if isinstance(arg, Return):
+                    yield arg.value
+                    continue
+                ret = func(*args, arg, **kwargs)
+                if isawaitable(ret):
+                    ret = await ret
+                yield ret
+    else:
+        sema = AsyncSemaphore(max_workers)
+        queue: AsyncQueue[None | AsyncFuture] = AsyncQueue()
+        get, put = queue.get, queue.put_nowait
+        async def call(args):
+            async with sema:
+                ret = func(*args, **kwargs)
+                if isawaitable(ret):
+                    ret = await ret
+                return ret
+        async def make_tasks():
+            try:
+                async for args in async_zip(it, *its):
+                    if arg_func is not None:
+                        arg = arg_func(*args, **kwargs)
+                        if isawaitable(arg):
+                            arg = await arg
+                        if isinstance(arg, Return):
+                            put(arg.value)
+                            continue
+                        args = (*args, arg)
+                    put(create_task(call(args)))
+            finally:
+                put(None)
+        async with TaskGroup() as tg:
+            create_task = tg.create_task
+            create_task(make_tasks())
+            while task := await get():
+                yield await task
 
 
 @overload
